use crate::cards::StandardCard as Card;

/// Used to manage BlackJack hands
#[derive(Clone, Debug, Default)]
pub(crate) struct Hand {
    pub(crate) cards: Vec<Card>,
}

impl Hand {
    pub(crate) fn new() -> Self {
        Self {
            cards: Vec::with_capacity(5),
        }
    }

    /// Add a card to hand
    pub(crate) fn add_card(&mut self, card: Card) {
        self.cards.push(card)
    }

    /// Calculate the score
    pub(crate) fn score(&self) -> u8 {
        let (mut ace_count, mut total) = self.cards.iter().fold((0u8, 0u8), |mut d, c| {
            let val = c.face().value();
            if val == 11 {
                d.0 += 1;
            } else {
                d.1 += val;
            }
            d
        });
        if ace_count >= 1 {
            if total <= 10 {
                total += 11;
            } else {
                total += 1;
            }
            ace_count -= 1;
        }
        total + ace_count
    }
}
