use crate::cards::StandardCard;
#[allow(deprecated)] // Deprecated for external use
use crate::deck_of_cards::STANDARD_DECK_OF_CARDS;
use rand::{thread_rng, Rng};
use std::error::Error as StdError;
use std::fmt::{Display, Formatter, Result as FmtResult};

/// Standard card deck
#[derive(Clone, Debug, Default)]
pub struct StandardDeck {
    /// Deck of cards
    pub cards: Vec<StandardCard>,
}

/// Deck error
#[derive(Debug, Clone, Copy)]
pub enum StandardDeckError {
    /// No cards left in the deck
    NoCard,
}

impl Display for StandardDeckError {
    fn fmt(&self, f: &mut Formatter) -> FmtResult {
        f.write_str(self.description())
    }
}

impl StdError for StandardDeckError {
    fn description(&self) -> &str {
        match *self {
            StandardDeckError::NoCard => "No Cards are left in the deck",
        }
    }
}

impl StandardDeck {
    /// Create a new deck
    /// sets: The number of sets to create
    /// None is the fastest way to create a deck size of 1
    /// Otherwise just pass an usize
    pub fn new<T: Into<Option<usize>>>(sets: T) -> Self {
        #[allow(deprecated)]
        let cards = match sets.into() {
            None => STANDARD_DECK_OF_CARDS.to_vec(),
            Some(sets) => {
                let mut vec = Vec::with_capacity(52 * sets);
                for _ in 0..sets {
                    vec.extend(STANDARD_DECK_OF_CARDS.iter())
                }
                vec
            }
        };
        Self { cards }
    }
    /// Draw a card from the deck at random
    pub fn draw(&mut self) -> Result<StandardCard, StandardDeckError> {
        let card_len = self.cards.len();
        let index = thread_rng().gen_range(0, card_len);
        // Games should never get to the point where the deck is empty
        if card_len + 1 > index {
            Ok(self.cards.remove(index))
        // But just in case...
        } else {
            Err(StandardDeckError::NoCard)
        }
    }
}
